import Vue from 'vue'
import Router from 'vue-router'
import Home from "../components/Home";
import Login from "../components/auth/Login";
import Register from "../components/auth/Register";
import Thanks from "../components/auth/Thanks";
import MyProfile from "../components/profile/MyProfile";
import AddWeb from "../components/webs/AddWeb";
import NotFound from "../components/NotFound";
import WebVerification from "../components/webs/WebVerification";
import WebDetail from "../components/webs/WebDetail";
import BasicDetail from "../components/crawl/BasicDetail";
import AddKeywords from "../components/webs/AddKeywords";
import KeywordsDetail from "../components/crawl/KeywordsDetail";
import CentralityDetail from "../components/crawl/CentralityDetail";
import VerifyEmail from "../components/auth/VerifyEmail";
import VerificationFailed from "../components/auth/VerificationFailed";
import Verified from "../components/auth/Verified";
import ForgottenPassword from "../components/auth/ForgottenPassword";
import ResetPass from "../components/auth/ResetPass";
import PasswordReset from "../components/auth/PasswordReset";
import DeleteWeb from "../components/webs/DeleteWeb";
import Contact from "../components/Contact";
import Tutorial from "../components/Tutorial";

Vue.use(Router)

export default new Router({
    mode: 'history',
    routes:[
        {
            path: '/',
            name: 'home',
            component: Home
        },
        {
            path: '/contact',
            name: 'contact',
            component: Contact
        },
        {
            path: '/tutorial',
            name: 'tutorial',
            component: Tutorial
        },
        {
            path: '/webs/add',
            name: 'add-website',
            component: AddWeb,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/verification_failed',
            name: 'verification_failed',
            component: VerificationFailed
        },
        {
            path: '/verified',
            name: 'verified',
            component: Verified
        },
        {
            path: '/forgotten_pass',
            name: 'forgotten_pass',
            component: ForgottenPassword
        },
        {
            path: '/password_reset',
            name: 'password_reset',
            component: PasswordReset
        },
        {
            path: '/reset_pass/:token',
            name: 'reset_pass',
            component: ResetPass
        },
        {
            path: '/verify/:token',
            name: 'verify',
            component: VerifyEmail
        },

        {
            path: '/login',
            name: 'login',
            component: Login
        },
        {
            path: '/register/successful',
            name: 'reg_success',
            component: Thanks
        },
        {
            path: '/register', //todo fix this
            component: Register
        },
        {
            path: '/profile/my_profile',
            name: 'profile',
            component: MyProfile,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/webs/:webID/verify',
            name: 'verify_web',
            component: WebVerification,
            props: true,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/webs/:webID/detail',
            name: 'web_detail',
            component: WebDetail,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/webs/:webID/addKeywords',
            name: 'add_keywords',
            component: AddKeywords,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/webs/:webID/deleteWeb',
            name: 'delete_web',
            component: DeleteWeb,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/crawl/:crawlID/basicdetail',
            name: 'basic_detail',
            component: BasicDetail,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/crawl/:crawlID/centralitydetail',
            name: 'centrality_detail',
            component: CentralityDetail,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '/crawl/:crawlID/keywordsdetail',
            name: 'keywords_detail',
            component: KeywordsDetail,
            meta : {
                requiresAuth: true
            }
        },
        {
            path: '*',
            component: NotFound
        }
    ]

})

