// store/index.js

import Vue from 'vue'
import Vuex from 'vuex'

// imports of AJAX functions will go here
import {  userDetail, authenticate, register, websitesList } from '@/api'
import { isValidJwt } from '@/utils'
import {getUsername} from "../utils";
import {addWebsite, webDetail} from "../api";

Vue.use(Vuex)

const state = {
    // single source of data
    user: {},
    jwt: {'token': localStorage.getItem('token')}
}

const actions = {
    // asynchronous operations

    login (context, userData) {
        context.commit('setUserData', { userData })
        return authenticate(userData)
            .then(response => context.commit('setJwtToken', { jwt: response.data }))
    },
    register (context, userData) {
        context.commit('setUserData', { userData })
        return register(userData)
            .then(context.dispatch('login', userData))
            .catch(error => {
                console.log('Error Registering: ', error)
                //EventBus.emit('failedRegistering: ', error)
            })
    },
    logout (context) { // wtf is context
        context.commit('unsetJwtToken')
    },

    userDetail (context) {
        return userDetail( context.state.jwt.token)
    },

    webDetail (context) {
        return webDetail( context.state.jwt.token)
    },

    websitesList (context) {
        return websitesList( context.state.jwt.token)
    },
    addWebsite (context, webData) {
        context.commit('setWebData', { webData })
        return addWebsite(context.state.jwt.token, webData)
    }
}

const mutations = {
    // isolated data mutations
    setUserData (state, payload) {
        state.userData = payload.userData
    },
    setWebData (state, payload) {
        state.webData = payload.webData
    },
    setJwtToken (state, payload) {
        localStorage.token = payload.jwt.token
        state.jwt = payload.jwt
    },
    unsetJwtToken (state){
        localStorage.token = ''
        state.jwt = ''
    }
}

const getters = {
    // reusable data accessors
    isAuthenticated (state) {
        return isValidJwt(state.jwt.token)
    },
    getUser (state){
        return getUsername(state.jwt.token)
    }
}

const store = new Vuex.Store({
    state,
    actions,
    mutations,
    getters
})

export default store
