import threading
from datetime import datetime
from flask import Flask,  request
from flask_cors import CORS
from auth_utils import token_required
from scripts.basicCrawler import basic_crawler
from scripts.centralityCrawler import centrality_crawler
from scripts.keywordsCrawler import keywords_crawler
from views.crawlers import crawl
from views.handlers import handleCrawlData

from views.profile import profile
from views.webs import webs
from views.auth import auth_bp, generate_confirmation_token
from models import db, User, Web, Crawler_data
from marshmallow_schemas.web import ma

#create app


app = Flask(__name__)


#app config
app.config['SQLALCHEMY_DATABASE_URI'] = 'postgresql://qaanggrz:2hOC-RWp2jgjAlcd6hQI3eHgI2u8zwdi@tai.db.elephantsql.com/qaanggrz'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

app.config['SECRET_KEY'] = "secretkey"



#emails

from flask_mail import Mail, Message



app.config['MAIL_SERVER']='smtp.gmail.com'
app.config['MAIL_PORT'] = 587
app.config['MAIL_USERNAME'] = 'betkyanalyzator@gmail.com'
app.config['MAIL_PASSWORD'] = 'aeyfcoiqxmdgfnye'
app.config['MAIL_USE_TLS'] = True
app.config['MAIL_USE_SSL'] = False

mail = Mail(app)


def handleCrawl(web,web_id, type, words = None):
    if type == 1:
        data = basic_crawler(web.url, web.token)
    with app.app_context():


        if type == 2:
            data = keywords_crawler(web.url, words, web.token)

        if type == 3:
            data = centrality_crawler(web.url, web.token)
            if data != 'not_verified':
                data = handleCrawlData(data)
        web = Web.query.filter_by(id=web_id).first()

        web.active_connection = False
        if data != 'not_verified':
            crawl = Crawler_data(
                data=data,
                date=datetime.now(),
                web_id=web_id,
                type_id=type)

            db.session.add(crawl)
        else:
            web.verified = False
        db.session.commit()

@app.route('/crawl/api/basic_crawl/<web_id>', methods = ['GET'])
@token_required
def crawl_basic(current_user, web_id):
    web = Web.query.filter_by(id =web_id).first()
    crawls = Crawler_data.query.filter_by(web_id = web_id).all()
    if len(crawls) >= 50:
        return 'too many crawls', 200
    web.active_connection = True
    db.session.commit()
    threading.Thread(target=handleCrawl, args=(web, web_id, 1)).start()
    return '', 200

@app.route('/crawl/api/keywords_crawl/<web_id>', methods = ['GET'])
@token_required
def keywords_basic(current_user, web_id):
    web = Web.query.filter_by(id =web_id).first()
    crawls = Crawler_data.query.filter_by(web_id=web_id).all()
    if len(crawls) >= 50:
        return 'too many crawls', 403
    web.active_connection = True
    db.session.commit()

    threading.Thread(target=handleCrawl, args=(web, web_id, 2,web.keywords['words'],)).start()
    return '', 200
@app.route('/crawl/api/centrality_crawl/<web_id>', methods = ['GET'])
@token_required
def centrality_crawl(current_user, web_id):
    web = Web.query.filter_by(id =web_id).first()
    crawls = Crawler_data.query.filter_by(web_id=web_id).all()
    if len(crawls) >= 50:
        return 'too many crawls', 403
    web.active_connection = True
    db.session.commit()
    threading.Thread(target=handleCrawl, args=(web, web_id, 3)).start()

    return '', 200
@app.route('/verify_email', methods=['POST'])
def mails():
    data = request.json

    address = data['email'].lower()
    print(address)
    try:
        user = User.query.filter_by(email=address).first()
    except:
        return "not in database"
    threading.Thread(target=send_mail, args=(address,)).start()
    return "Message sent!"


@app.route('/forgotten_password', methods=['POST'])
def forgotten_pass(**kwargs):
    data = request.json
    email = data['email'].lower()

    try:
        user = User.query.filter_by(email=email).first()
        threading.Thread(target=send_mail_forg, args=(user.email,)).start()
        return '', 201
    except:
        pass



def send_mail(address):
    with app.app_context():

        msg = Message('Registration on server BEAN', sender='admin@bean.com', recipients=[address])
        token = generate_confirmation_token(address)
        url = 'https://be-an.herokuapp.com/verify/' + token
        msg.html = "Please, verify your email with this link: <a href=\"" + url + "\">Verify!</a>"
        mail.send(msg)

def send_mail_forg(address):
    with app.app_context():

        msg = Message('Forgotten password', sender='admin@bean.com', recipients=[address])
        token = generate_confirmation_token(address)
        url = 'https://be-an.herokuapp.com/reset_pass/' + token
        msg.html = "Reset your password on this link: <a href=\"" + url + "\">Reset!</a>"
        mail.send(msg)

#init utils
db.init_app(app)
ma.init_app(app)


#CORS setting
cors = CORS(app, resources={r"/*": {"origins": "*"}})

#Register blueprints
app.register_blueprint(auth_bp)
app.register_blueprint(profile)
app.register_blueprint(webs)
app.register_blueprint(crawl)


if __name__ == "__main__":
    app.run(debug=True)

