from functools import wraps
import jwt
from flask import jsonify, request, current_app

from models import User


def token_required(f):
    @wraps(f)
    def _verify(*args, **kwargs):
        auth_headers = request.headers.get('Authorization', '').split()

        invalid_msg = {
            'message': 'Invalid token. Registeration and / or authentication required',
            'authenticated': False
        }
        expired_msg = {
            'message': 'Expired token. Reauthentication required.',
            'authenticated': False
        }

        if len(auth_headers) != 2:
            return jsonify(invalid_msg), 401

        try:
            token = auth_headers[1]
            data = jwt.decode(jwt = token, key = current_app.config['SECRET_KEY'], algorithms= ["HS256"])
            user = User.query.filter_by(email=data['sub']).first()
            if not user:
                raise RuntimeError('User not found')
            return f(current_user =user, *args, **kwargs)
        except jwt.ExpiredSignatureError:

            return jsonify(expired_msg), 401
        except (jwt.InvalidTokenError, Exception) as e:
            return jsonify(invalid_msg), 401

    return _verify