import re
import sys
import time
from deepmerge import always_merger
import requests
from urllib.parse import urlparse
import urllib3
from langdetect import detect
import readtime

urllib3.disable_warnings()


def validate(seed):
    source = requests.get(seed, verify=False).text
    soup = BeautifulSoup(source, 'html.parser')
    try:
        token = soup.find("meta", property="betkyanalyzertoken")
        token = token["content"]
    except:
        token = None
    return token


from bs4 import BeautifulSoup

def domain(seed):
    domain = urlparse(seed).netloc
    if seed.startswith('https'):
        return 'https://' + domain + '/'
    else:
        return 'http://' + domain + '/'


def analyze_link(link, seed):
    if link.endswith(('.jpg', '.jpeg', '.svg', '.png', '.pdf')):
        return 'none'
    elif link.startswith(('mailto:','tel:')):
        return 'none'
    elif ("www." in str(link) or 'http://' in str(link) or 'https://' in str(link)) and domain(seed) not in str(link):
        return 'outsider'
    elif domain(seed) in str(link):
        return 'full_url'
    else:
        return 'inner'


def connect(link, seed):
    pass


def basic_crawler(seed, token):
    if seed[-1] != '/':
        seed = seed + '/'
    https = False
    if seed.startswith('https'):
        https = True
    frontier = [seed]
    crawled = []
    data = {}
    ret_token = validate(seed)
    if ret_token != token:
        return 'not_verified'
    outiders = []
    inner = []
    response_times = []
    errors = []
    top_langs = []
    robots_file = False
    sitemaps_file = False
    title_graph = [['pages with title', 0],['pages without title', 0]]
    description_graph = [['pages with description', 0],['pages without description', 0]]
    no_desc = 0
    no_titl = 0

    start = time.time()
    while frontier:

        page = frontier.pop()
        if page not in crawled:
            print('Crawling:' + page)
            sys.stdout.flush()
            crawled.append(page)
            try:
                if page == seed:
                    try:
                        robots = seed + 'robots.txt'

                        rob = requests.get(robots, verify=False)
                        print(rob.headers["content-type"])
                        if rob.status_code == 200 and (rob.headers["content-type"].split(';'))[0] == 'text/plain':
                            robots_file = True
                    except:
                        pass
                    try:
                        sitemap = seed + 'sitemap.xml'

                        sit = requests.get(sitemap, verify=False)
                        print((sit.headers["content-type"].split(';'))[0])
                        if sit.status_code == 200 and (
                                (sit.headers["content-type"].split(';'))[0] == 'application/xslt+xml' or
                                (sit.headers["content-type"].split(';'))[0] == 'application/xml' or
                                (sit.headers["content-type"].split(';'))[0] == 'text/xml'):
                            sitemaps_file = True
                    except Exception as e:
                        print(e)
                        errors.append(e)


                source = requests.get(page, verify=False)

                selenium_source = requests.post('https://mysterious-cove-53924.herokuapp.com/selenium', json={'page': page})

                response_time = source.elapsed.total_seconds()
                page_name = page.replace(seed, '')
                if page_name == '':
                    page_name = '/'
                response_times.append([page_name,response_time])

                source.encoding = source.apparent_encoding
                print(source.headers["content-type"])
                if "text/html" not in source.headers["content-type"]:
                    continue
                source = source.text

                soup = BeautifulSoup(selenium_source.text, 'html.parser')
                # soup = BeautifulSoup(source, 'html.parser')
                links = soup.findAll('a', href=True)

                for link in links:
                    link_type = analyze_link(link['href'], seed)
                    if link_type == "outsider" and link['href'] not in outiders:
                        outiders.append(link['href'])
                    elif (link_type == "full_url" or link_type == "inner") and link['href'] not in inner:
                        inner.append(link['href'])

                    if link_type == 'full_url':
                        frontier.append(link['href'])
                    elif link_type == 'inner':
                        if link['href'] == '/':
                            continue
                        if link['href'] != '' and link['href'][0] == '/':
                            link['href'] = link['href'][1:]
                        frontier.append(domain(seed) + link['href'])
                    else:
                        continue

                page_data = {}
                page_data = always_merger.merge(page_data, {'response_time': response_time})

                # jazyk stranky
                lang = detect(soup.get_text())
                top_langs.append(lang)
                page_data = always_merger.merge(page_data, {'lang': lang})

                # readtime
                tag = soup.body
                body_text = ''
                for string in tag.strings:
                    string = re.sub(' +', ' ', string)
                    body_text = body_text + string
                result = readtime.of_text(body_text)
                page_data = always_merger.merge(page_data, {'readtime': result.seconds})

                # h1
                try:
                    main_title = soup.h1.text
                except:
                    main_title = None

                if main_title == None:
                    main_title_len = 0
                else:
                    main_title_len = len(main_title)
                page_data = always_merger.merge(page_data, {'h1':
                                                                {'value': main_title, 'lenght': main_title_len}
                                                            })

                #images

                images = soup.find_all('img')
                for img in images:
                    try:
                        alt = img.get('alt', '')
                        if alt == '':
                            alt = img.get('data-alt', '')
                        src = img.get('src', '')
                        if src == '':
                            src = img.get('data-src', '')
                        src_name = src.split('/')[-1]
                        format = src.split('.')[-1]
                        page_data = always_merger.merge(page_data, {'images': {src_name.split('.')[0]: {'format': format, 'alt': alt} }})
                    except:
                        continue

                # title
                try:
                    title = soup.title.text
                except:
                    title = None
                    no_titl = no_titl + 1
                if title == None:
                    title_len = 0
                    title_graph[1][1] = title_graph[1][1] + 1
                else:
                    title_len = len(title)
                    title_graph[0][1] = title_graph[0][1] + 1
                page_data = always_merger.merge(page_data,
                                                {'title': {'value': title, 'lenght': title_len}})  # none nedelat delku

                # description
                try:
                    desc = soup.find("meta", attrs={'name': 'description'})
                    desc = desc['content']
                except:
                    desc = None
                    no_desc = no_desc + 1
                if desc == None:
                    desc_len = 0
                    description_graph[1][1] = description_graph[1][1] + 1
                else:
                    desc_len = len(desc)
                    description_graph[0][1] = description_graph[0][1] + 1
                page_data = always_merger.merge(page_data,
                                                {'description': {'value': desc, 'lenght': desc_len}
                                                 })

                data = always_merger.merge(data, {page: page_data})

                time.sleep(1)
                elapsed_time = (time.time() - start)
                print(elapsed_time)
                if elapsed_time > 14400:
                    break
            except Exception as e:
                print('Failed to crawl ', page, '. Error: ', e)
                errors.append(e)
                time.sleep(5)

    data = always_merger.merge(data, {
        'top': {'langs': top_langs, 'no_t': no_titl, 'no_d': no_desc, 'sitemap': sitemaps_file, 'robots': robots_file,
                'inner': len(inner), 'outsider': len(outiders), 'https': https,
                'request_times': response_times, 'title_graph': title_graph, 'desc_graph': description_graph}})

    return data