import time
import requests
from urllib.parse import urlparse
import urllib3
import networkx as nx
from networkx import in_degree_centrality, closeness_centrality, betweenness_centrality
urllib3.disable_warnings()


def validate(seed):
    source = requests.get(seed, verify=False).text
    soup = BeautifulSoup(source, 'html.parser')
    try:
        token = soup.find("meta", property="betkyanalyzertoken")
        token = token["content"]
    except:
        token = None

    return token


from bs4 import BeautifulSoup


def domain(seed):
    domain = urlparse(seed).netloc
    if seed.startswith('https'):
        return 'https://' + domain + '/'
    else:
        return 'http://' + domain + '/'


def analyze_link(link, seed):

    if link.endswith(('.jpg', '.jpeg', '.svg', '.png', '.pdf')):
        return 'none'
    elif link.startswith(('mailto:', 'tel:')):
        return 'none'
    elif ("www." in str(link) or 'http://' in str(link) or 'https://' in str(link)) and domain(seed) not in str(link):
        return 'outsider'
    elif domain(seed) in str(link):
        return 'full_url'
    else:
        return 'inner'


def centrality_crawler(seed, token):
    if seed[-1] != '/':
        seed = seed + '/'
    print('seed', seed)
    frontier = [seed]
    crawled = []
    data = {}
    ret_token = validate(seed)
    if ret_token != token:
        return 'not_verified'
    nodes = []
    links_graph = []
    id = 1
    start = time.time()
    while frontier:
        page = frontier.pop()
        if page not in crawled:
            print('Crawling:' + page)
            crawled.append(page)
            page_links = []
            try:

                source = requests.get(page, verify=False)
                source.encoding = source.apparent_encoding
                print(source.headers["content-type"])
                if "text/html" not in source.headers["content-type"]:
                    continue
                source = source.text
                selenium_source = requests.post('https://mysterious-cove-53924.herokuapp.com/selenium', json={'page': page})

                soup = BeautifulSoup(selenium_source.text, 'html.parser')

                links = soup.findAll('a', href=True)
                node = {'id': id, 'name': page, '_color': 'red'}
                id = id + 1
                nodes.append(node)
                print('page', page)
                for link in links:
                    link_type = analyze_link(link['href'], seed)

                    if link_type == 'full_url':
                        frontier.append(link['href'])
                        page_links.append(link['href'])
                    elif link_type == 'inner':
                        if link['href'] == '/':
                            continue
                        if link['href'] != '' and link['href'][0] == '/':
                            link['href'] = link['href'][1:]
                        frontier.append(domain(seed) + link['href'])
                        page_links.append(domain(seed) + link['href'])
                    else:
                        continue

                links_graph.append({page: page_links})

                time.sleep(1)
                elapsed_time = (time.time() - start)
                if elapsed_time > 14400:
                    break

            except Exception as e:
                print('Failed to crawl ', page, '. Error: ', e)
                time.sleep(1)
    links = graph(links_graph, nodes)
    deg, bet, clos = graph_cent(links, nodes)

    data = {'nodes': nodes, 'links': links, 'degree': deg, 'bet': bet, 'clos':clos}
    return data


def graph(links_graph, nodes):
    links_final = []
    id = 0
    for x in links_graph:
        page = list(x.keys())[0]
        for n in nodes:
            if n['name'] == page:
                id = n['id']
                break
        for y in x[page]:
            id_to = 0
            for n in nodes:
                if n['name'] == y:
                    id_to = n['id']
                    break
            if id_to == 0:
                continue
            links_final.append({'sid': id, 'tid': id_to, '_color': 'black'})
    return links_final

def graph_cent(links, nodes):
    G = nx.DiGraph()
    for n in nodes:
        G.add_node(n['name'])
    for l in links:
        _from = ''
        to = ''
        for x in nodes:
            if l['sid'] == x['id']:
                _from = x['name']
            if l['tid'] == x['id']:
                to = x['name']
            if _from != '' and to != '':
                break
        G.add_edge(_from, to)

    return in_degree_centrality(G), betweenness_centrality(G), closeness_centrality(G)