import threading
import unidecode
import requests
from flask import Blueprint
from datetime import datetime, timedelta
import jwt
from flask import jsonify, request, current_app
from itsdangerous import URLSafeTimedSerializer
from werkzeug.security import generate_password_hash, check_password_hash

from models import db, User

auth_bp = Blueprint('auth', __name__)

def generate_confirmation_token(email):
    serializer = URLSafeTimedSerializer(current_app.config["SECRET_KEY"], salt="email-confirm")
    return serializer.dumps(email)

def confirm_token(token, expiration=360):
    serializer = URLSafeTimedSerializer( current_app.config["SECRET_KEY"], salt="email-confirm")
    try:
        email = serializer.loads(
            token,
            max_age=expiration
        )
    except:
        return 'expired'
    return email

@auth_bp.route('/api/verify_finish/<token>', methods=['GET'])
def verify_finish(token,**kwargs):
    email = confirm_token(token).lower()
    if email == 'expired':
        return 'expired'
    else:
        try:
            user = User.query.filter_by(email = email).first()
            if user.confirmed == True:
                return 'already confirmed'
            else:
                user.confirmed = True
                db.session.commit()
                return 'success'
        except:
            return 'expired'


@auth_bp.route('/check_confirmed', methods=['POST'])
def check_confirmed(**kwargs):
    data = request.json
    email = data['email'].lower()
    try:
        user = User.query.filter_by(email = email).first()
        if user.confirmed == False:
            return 'not verified', 201
        else:
            return 'ver', 201
    except:
        return 'failed', 201
@auth_bp.route('/login', methods=['POST'])
def login(**kwargs):

    data = request.json
    email = data['email'].lower()
    password = data['password']

    try:
        user = User.query.filter_by(email = email).first()
    except:
        return 'failed', 201
    if user and check_password_hash(user.password, password):
            token = jwt.encode({
                'sub': user.email.lower(),
                'iat': datetime.utcnow(),
                'exp': datetime.utcnow() + timedelta(minutes=1440)},
                current_app.config['SECRET_KEY'])
            print(token)
            return jsonify({'token': token}), 201
    else:
        return 'failed'
def handle_mail(email):
    requests.post('https://be-an.herokuapp.com/verify_email', json={"email": email})


@auth_bp.route('/pass_reset', methods = ['POST'])
def pass_reset(**kwargs):
    data = request.json
    new_pass = data['new_pass']
    email = confirm_token(data['token']).lower()
    if email == 'expired':
        return 'expired', 201
    user = User.query.filter_by(email=email).first()
    user.password = generate_password_hash(new_pass)
    db.session.commit()
    return '', 201

@auth_bp.route('/register', methods = ['POST'])
def register(**kwargs):
    data = request.json
    first = unidecode.unidecode(data['firstName'])
    last = unidecode.unidecode(data['lastName'])

    user = User(email=data['email'].lower(),
                password = generate_password_hash(data['password']),
                first_name = first,
                last_name = last,
                confirmed = False)
    try:
        db.session.add(user)
        db.session.commit()
        threading.Thread(target=handle_mail, args=(user.email,)).start()
        return '', 201
    except Exception as e:
        if "duplicate key value violates unique constraint" in str(e):
            return 'email not unique', 201
        return str(e), 500
