import uuid
import re
from flask import Blueprint
from flask import request
from sqlalchemy.orm.attributes import flag_modified
from marshmallow_schemas.web import web_schema
from auth_utils import token_required
from models import db, Web, Crawler_data
from scripts.verficationScript import verification_crawler
import unidecode


webs = Blueprint('webs', __name__, url_prefix='/webs')

@webs.route('/api/<web_id>/active_connection', methods= ['GET'])
@token_required
def active_connection(current_user,web_id):
    web = Web.query.filter_by(id=web_id).first()
    if web.active_connection == True:
        return 'true', 200
    else:
        return 'false', 200

@webs.route('/api/<web_id>/detail', methods= ['GET'])
@token_required
def web_detail(current_user,web_id):

    web = Web.query.filter_by(id =web_id).first()
    web_data = web_schema.dump(web)

    return web_data, 200


@webs.route('/api/<web_id>/delete_crawls', methods= ['POST'])
@token_required
def delete_crawls(current_user,web_id):
    data = request.json
    for crawl in data['data']:
        crawl_to_delete = Crawler_data.\
                            query.filter_by(id=crawl['id'])\
                            .first()
        db.session.delete(crawl_to_delete)
    db.session.commit()
    return '', 201


@webs.route('/api/<web_id>/delete_web', methods= ['GET'])
@token_required
def delete_web(current_user,web_id, *args,**kwargs):
    crawls = Crawler_data.query.filter_by(web_id = web_id).all()
    for c in crawls:
        db.session.delete(c)
    web = Web.query.filter_by(id=web_id).first()
    db.session.delete(web)
    db.session.commit()
    return '', 200

@webs.route('/api/verification/<id>', methods= ['GET'])
@token_required
def web_verification(current_user,id, *args,**kwargs):
    web_id = id
    web = Web.query.filter_by(id=web_id).first()
    web_data = web_schema.dump(web)
    ver = verification_crawler(web_data['url'], web_data['token'])
    if ver:
        web.verified = True
    else:
        web.verified = False
    db.session.commit()

    return str(ver), 200

@webs.route('/api/<web_id>/return_keywords', methods= ['GET'])
@token_required
def return_keywords(current_user,web_id, *args,**kwargs):
    web = Web.query.filter_by(id=web_id).first()
    try:
        data = web.keywords['words']
    except:
        data = []

    return {'words': data}, 200

@webs.route('/api/<web_id>/delete_keywords', methods= ['POST'])
@token_required
def delete_keywords(current_user,web_id, *args,**kwargs):
    web = Web.query.filter_by(id=web_id).first()
    data = request.json
    for w in web.keywords['words']:
        if w in data['data']:

            web.keywords['words'].remove(w)

    flag_modified(web, 'keywords')
    db.session.commit()

    return {'words': data}, 200

@webs.route('/api/<web_id>/add_keywords', methods= ['POST'])
@token_required
def add_keywords(current_user,web_id, *args,**kwargs):
    web = Web.query.filter_by(id=web_id).first()
    try:
        new_words = web.keywords['words']
    except:
        new_words = []

    data = request.json
    words = data['data'].split(',')
    web.keywords = {}
    for word in words:
        word = re.sub(' +', ' ', word)
        try:
            if word[0] == ' ':
                word = word[1:]
            if word[-1] == ' ':
                word = word[:-1]
            if word != None and word not in new_words:
                new_words.append(word)
        except:
            continue
    web.keywords = dict({'words': new_words})
    flag_modified(web, 'keywords')
    db.session.commit()

    return '', 201

@webs.route('/api/add', methods= ['POST'])
@token_required
def add_web(current_user):
    webs = Web.query.filter_by(user_id=current_user.id).all()
    if len(webs) >= 10:
        return 'too many websites', 201
    data = request.json

    token = uuid.uuid4()
    print(current_user.id)
    name = unidecode.unidecode(data['name'])
    web = Web(name=name,
                url=data['url'],
                token=token,
                user_id=current_user.id,
                verified=False)

    db.session.add(web)
    db.session.commit()

    return {'webID': web.id}, 201


@webs.route('/api/all', methods= ['GET'])
@token_required
def webs_all(current_user):
    web_data = []
    web_list = Web.query.filter_by(user_id = current_user.id).all()
    for web in web_list:
        web = {'id' : web.id, 'name': web.name, 'url': web.url, 'token': web.token, 'verified': web.verified}
        web_data.append(web)

    data = {'websites': web_data}

    return data